@@ -17,13 +17,16 @@
     provides methods and functions to convert SAML classes to and from strings.
 """
 
-
 import logging
+
 import six
+
+from saml2.version import version as __version__
 from saml2.validate import valid_instance
 
 try:
     from xml.etree import cElementTree as ElementTree
+
     if ElementTree.VERSION < '1.3.0':
         # cElementTree has no support for register_namespace
         # neither _namespace_map, thus we sacrify performance
@@ -34,22 +37,23 @@ except ImportError:
         import cElementTree as ElementTree
     except ImportError:
         from elementtree import ElementTree
+import defusedxml.ElementTree
 
-root_logger = logging.getLogger(__name__)
-root_logger.level = logging.NOTSET
+
+logger = logging.getLogger(__name__)
 
 NAMESPACE = 'urn:oasis:names:tc:SAML:2.0:assertion'
-#TEMPLATE = '{urn:oasis:names:tc:SAML:2.0:assertion}%s'
-#XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'
+# TEMPLATE = '{urn:oasis:names:tc:SAML:2.0:assertion}%s'
+# XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'
 
 NAMEID_FORMAT_EMAILADDRESS = (
     "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress")
 
 # These are defined in saml2.saml
-#NAME_FORMAT_UNSPECIFIED = (
+# NAME_FORMAT_UNSPECIFIED = (
 #    "urn:oasis:names:tc:SAML:2.0:attrname-format:unspecified")
-#NAME_FORMAT_URI = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri"
-#NAME_FORMAT_BASIC = "urn:oasis:names:tc:SAML:2.0:attrname-format:basic"
+# NAME_FORMAT_URI = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri"
+# NAME_FORMAT_BASIC = "urn:oasis:names:tc:SAML:2.0:attrname-format:basic"
 
 DECISION_TYPE_PERMIT = "Permit"
 DECISION_TYPE_DENY = "Deny"
@@ -85,7 +89,7 @@ def create_class_from_xml_string(target_class, xml_string):
     """
     if not isinstance(xml_string, six.binary_type):
         xml_string = xml_string.encode('utf-8')
-    tree = ElementTree.fromstring(xml_string)
+    tree = defusedxml.ElementTree.fromstring(xml_string)
     return create_class_from_element_tree(target_class, tree)
 
 
@@ -267,7 +271,7 @@ class ExtensionElement(object):
 
 
 def extension_element_from_string(xml_string):
-    element_tree = ElementTree.fromstring(xml_string)
+    element_tree = defusedxml.ElementTree.fromstring(xml_string)
     return _extension_element_from_element_tree(element_tree)
 
 
@@ -289,7 +293,6 @@ def _extension_element_from_element_tree(element_tree):
 
 
 class ExtensionContainer(object):
-
     c_tag = ""
     c_namespace = ""
 
@@ -299,6 +302,7 @@ class ExtensionContainer(object):
         self.text = text
         self.extension_elements = extension_elements or []
         self.extension_attributes = extension_attributes or {}
+        self.encrypted_assertion = None
 
     # Three methods to create an object from an ElementTree
     def harvest_element_tree(self, tree):
@@ -402,7 +406,7 @@ def make_vals(val, klass, klass_inst=None, prop=None, part=False,
     """
     cinst = None
 
-    #print("make_vals(%s, %s)" % (val, klass))
+    # print("make_vals(%s, %s)" % (val, klass))
 
     if isinstance(val, dict):
         cinst = klass().loadd(val, base64encode=base64encode)
@@ -570,15 +574,16 @@ class SamlBase(ExtensionContainer):
 
     def tag_get_uri(self, elem):
         if elem.tag[0] == "{":
-                uri, tag = elem.tag[1:].split("}")
-                return uri
+            uri, tag = elem.tag[1:].split("}")
+            return uri
         return None
 
     def get_ns_map(self, elements, uri_set):
 
         for elem in elements:
             uri_set = self.get_ns_map_attribute(elem.attrib, uri_set)
-            uri_set = self.get_ns_map(elem.getchildren(), uri_set)
+            children = list(elem)
+            uri_set = self.get_ns_map(children, uri_set)
             uri = self.tag_get_uri(elem)
             if uri is not None:
                 uri_set.add(uri)
@@ -591,12 +596,17 @@ class SamlBase(ExtensionContainer):
             prefix_map["encas%d" % len(prefix_map)] = uri
         return prefix_map
 
-    def get_xml_string_with_self_contained_assertion_within_advice_encrypted_assertion(self, assertion_tag, advice_tag):
-        for tmp_encrypted_assertion in self.assertion.advice.encrypted_assertion:
+    def get_xml_string_with_self_contained_assertion_within_advice_encrypted_assertion(
+            self, assertion_tag, advice_tag):
+        for tmp_encrypted_assertion in \
+                self.assertion.advice.encrypted_assertion:
             if tmp_encrypted_assertion.encrypted_data is None:
-                prefix_map = self.get_prefix_map([tmp_encrypted_assertion._to_element_tree().find(assertion_tag)])
+                prefix_map = self.get_prefix_map([
+                    tmp_encrypted_assertion._to_element_tree().find(
+                        assertion_tag)])
                 tree = self._to_element_tree()
-                encs = tree.find(assertion_tag).find(advice_tag).findall(tmp_encrypted_assertion._to_element_tree().tag)
+                encs = tree.find(assertion_tag).find(advice_tag).findall(
+                    tmp_encrypted_assertion._to_element_tree().tag)
                 for enc in encs:
                     assertion = enc.find(assertion_tag)
                     if assertion is not None:
@@ -604,17 +614,23 @@ class SamlBase(ExtensionContainer):
 
         return ElementTree.tostring(tree, encoding="UTF-8").decode('utf-8')
 
-    def get_xml_string_with_self_contained_assertion_within_encrypted_assertion(self, assertion_tag):
-        """ Makes a encrypted assertion only containing self contained namespaces.
+    def get_xml_string_with_self_contained_assertion_within_encrypted_assertion(
+            self, assertion_tag):
+        """ Makes a encrypted assertion only containing self contained
+        namespaces.
 
         :param assertion_tag: Tag for the assertion to be transformed.
         :return: A new samlp.Resonse in string representation.
         """
-        prefix_map = self.get_prefix_map([self.encrypted_assertion._to_element_tree().find(assertion_tag)])
+        prefix_map = self.get_prefix_map(
+            [self.encrypted_assertion._to_element_tree().find(assertion_tag)])
 
         tree = self._to_element_tree()
 
-        self.set_prefixes(tree.find(self.encrypted_assertion._to_element_tree().tag).find(assertion_tag), prefix_map)
+        self.set_prefixes(
+            tree.find(
+                self.encrypted_assertion._to_element_tree().tag).find(
+                assertion_tag), prefix_map)
 
         return ElementTree.tostring(tree, encoding="UTF-8").decode('utf-8')
 
@@ -632,7 +648,7 @@ class SamlBase(ExtensionContainer):
 
         # fixup all elements in the tree
         memo = {}
-        for elem in elem.getiterator():
+        for elem in elem.iter():
             self.fixup_element_prefixes(elem, uri_map, memo)
 
     def fixup_element_prefixes(self, elem, uri_map, memo):
@@ -647,6 +663,7 @@ class SamlBase(ExtensionContainer):
                     new_name = uri_map[uri] + ":" + tag
                     memo[name] = new_name
                     return new_name
+
         # fix element name
         name = fixup(elem.tag)
         if name:
@@ -723,7 +740,7 @@ class SamlBase(ExtensionContainer):
                 childs.append(member)
         return childs
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def set_text(self, val, base64encode=False):
         """ Sets the text property of this instance.
 
@@ -732,7 +749,7 @@ class SamlBase(ExtensionContainer):
         :return: The instance
         """
 
-        #print("set_text: %s" % (val,))
+        # print("set_text: %s" % (val,))
         if isinstance(val, bool):
             if val:
                 setattr(self, "text", "true")
@@ -764,22 +781,20 @@ class SamlBase(ExtensionContainer):
         """
 
         for prop, _typ, _req in self.c_attributes.values():
-            #print("# %s" % (prop))
             if prop in ava:
-                if isinstance(ava[prop], bool):
-                    setattr(self, prop, "%s" % ava[prop])
-                elif isinstance(ava[prop], int):
-                    setattr(self, prop, "%d" % ava[prop])
+                value = ava[prop]
+                if isinstance(value, (bool, int)):
+                    setattr(self, prop, str(value))
                 else:
-                    setattr(self, prop, ava[prop])
+                    setattr(self, prop, value)
 
         if "text" in ava:
             self.set_text(ava["text"], base64encode)
 
         for prop, klassdef in self.c_children.values():
-            #print("## %s, %s" % (prop, klassdef))
+            # print("## %s, %s" % (prop, klassdef))
             if prop in ava:
-                #print("### %s" % ava[prop])
+                # print("### %s" % ava[prop])
                 # means there can be a list of values
                 if isinstance(klassdef, list):
                     make_vals(ava[prop], klassdef[0], self, prop,
@@ -807,9 +822,7 @@ class SamlBase(ExtensionContainer):
                 self.text = None
 
     def __eq__(self, other):
-        try:
-            assert isinstance(other, SamlBase)
-        except AssertionError:
+        if not isinstance(other, SamlBase):
             return False
 
         self.clear_text()
@@ -978,7 +991,7 @@ def extension_elements_to_elements(extension_elements, schemas):
     if isinstance(schemas, list):
         pass
     elif isinstance(schemas, dict):
-        schemas = schemas.values()
+        schemas = list(schemas.values())
     else:
         return res
 
@@ -1003,3 +1016,19 @@ def extension_elements_as_dict(extension_elements, onts):
         except KeyError:
             res[elem.c_tag] = [elem]
     return res
+
+
+REQUIRED = 2
+
+
+def is_required_attribute(cls, attr):
+    """
+    Check if the attribute is a required attribute for a specific SamlBase
+    class.
+
+    :param cls: The class
+    :param attr: An attribute, note it must be the name of the attribute
+        that appears in the XSD in which the class is defined.
+    :return: True if required
+    """
+    return cls.c_attributes[attr][REQUIRED]
