@@ -1,4 +1,5 @@
-from six.moves.urllib.parse import urlencode, parse_qs, urlparse
+from six.moves.urllib import parse
+
 from saml2.entity import Entity
 from saml2.response import VerificationError
 
@@ -9,34 +10,40 @@ IDPDISC_POLICY = "urn:oasis:names:tc:SAML:profiles:SSO:idp-discovery-protocol:si
 
 class DiscoveryServer(Entity):
     def __init__(self, config=None, config_file=""):
-        Entity.__init__(self, "disco", config, config_file)
+        if config or config_file:
+            Entity.__init__(self, "disco", config, config_file)
 
     def parse_discovery_service_request(self, url="", query=""):
         if url:
-            part = urlparse(url)
-            dsr = parse_qs(part[4])
+            part = parse.urlparse(url)
+            dsr = parse.parse_qs(part[4])
         elif query:
-            dsr = parse_qs(query)
+            dsr = parse.parse_qs(query)
         else:
             dsr = {}
 
         # verify
 
-        for key in ["isPassive", "return", "returnIDParam", "policy"]:
+        for key in ["isPassive", "return", "returnIDParam", "policy", 'entityID']:
             try:
-                assert len(dsr[key]) == 1
+                if len(dsr[key]) != 1:
+                    raise Exception("Invalid DS request keys: {k}".format(k=key))
                 dsr[key] = dsr[key][0]
             except KeyError:
                 pass
 
         if "return" in dsr:
-            part = urlparse(dsr["return"])
+            part = parse.urlparse(dsr["return"])
             if part.query:
-                qp = parse_qs(part.query)
+                qp = parse.parse_qs(part.query)
                 if "returnIDParam" in dsr:
-                    assert dsr["returnIDParam"] not in qp.keys()
+                    if dsr["returnIDParam"] in qp.keys():
+                        raise Exception(
+                            "returnIDParam value should not be in the query params"
+                        )
                 else:
-                    assert "entityID" not in qp.keys()
+                    if "entityID" in qp.keys():
+                        raise Exception("entityID should not be in the query params")
         else:
             # If metadata not used this is mandatory
             raise VerificationError("Missing mandatory parameter 'return'")
@@ -44,10 +51,13 @@ class DiscoveryServer(Entity):
         if "policy" not in dsr:
             dsr["policy"] = IDPDISC_POLICY
 
-        try:
-            assert dsr["isPassive"] in ["true", "false"]
-        except KeyError:
-            pass
+        is_passive = dsr.get("isPassive")
+        if is_passive not in ["true", "false"]:
+            raise ValueError(
+                "Invalid value '{v}' for attribute '{attr}'".format(
+                    v=is_passive, attr="isPassive"
+                )
+            )
 
         if "isPassive" in dsr and dsr["isPassive"] == "true":
             dsr["isPassive"] = True
@@ -69,9 +79,9 @@ class DiscoveryServer(Entity):
             return_url = kwargs["return"]
 
         if entity_id:
-            qp = urlencode({returnIDParam: entity_id})
+            qp = parse.urlencode({returnIDParam: entity_id})
 
-            part = urlparse(return_url)
+            part = parse.urlparse(return_url)
             if part.query:
                 # Iff there is a query part add the new info at the end
                 return_url = "%s&%s" % (return_url, qp)
@@ -90,10 +100,6 @@ class DiscoveryServer(Entity):
 
     def verify_return(self, entity_id, return_url):
         for endp in self.metadata.discovery_response(entity_id):
-            try:
-                assert return_url.startswith(endp["location"])
-            except AssertionError:
-                pass
-            else:
+            if not return_url.startswith(endp["location"]):
                 return True
         return False
