@@ -8,16 +8,16 @@ import six
 from six.moves.urllib.parse import quote, parse_qs
 from six.moves.http_cookies import SimpleCookie
 
-from saml2 import BINDING_HTTP_ARTIFACT, SAMLError
+from saml2 import BINDING_HTTP_ARTIFACT
 from saml2 import BINDING_HTTP_REDIRECT
 from saml2 import BINDING_HTTP_POST
 from saml2 import BINDING_URI
 from saml2 import BINDING_SOAP
+from saml2 import SAMLError
 from saml2 import time_util
 
 __author__ = 'rohe0002'
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -55,16 +55,28 @@ class Response(object):
 
     def _response(self, message="", **argv):
         if self.template:
-            return [self.template % message]
+            message = self.template % message
         elif self.mako_lookup and self.mako_template:
             argv["message"] = message
             mte = self.mako_lookup.get_template(self.mako_template)
-            return [mte.render(**argv)]
+            message = mte.render(**argv)
+
+        if isinstance(message, six.string_types):
+            return [message.encode('utf-8')]
+        elif isinstance(message, six.binary_type):
+            return [message]
         else:
-            if isinstance(message, six.string_types):
-                return [message]
-            else:
-                return message
+            return message
+
+    def add_header(self, ava):
+        """
+        Does *NOT* replace a header of the same type, just adds a new
+        :param ava: (type, value) tuple
+        """
+        self.headers.append(ava)
+
+    def reply(self, **kwargs):
+        return self.response(self.message, **kwargs)
 
 
 class Created(Response):
@@ -73,8 +85,8 @@ class Created(Response):
 
 class Redirect(Response):
     _template = '<html>\n<head><title>Redirecting to %s</title></head>\n' \
-        '<body>\nYou are being redirected to <a href="%s">%s</a>\n' \
-        '</body>\n</html>'
+                '<body>\nYou are being redirected to <a href="%s">%s</a>\n' \
+                '</body>\n</html>'
     _status = '302 Found'
 
     def __call__(self, environ, start_response, **kwargs):
@@ -86,8 +98,8 @@ class Redirect(Response):
 
 class SeeOther(Response):
     _template = '<html>\n<head><title>Redirecting to %s</title></head>\n' \
-        '<body>\nYou are being redirected to <a href="%s">%s</a>\n' \
-        '</body>\n</html>'
+                '<body>\nYou are being redirected to <a href="%s">%s</a>\n' \
+                '</body>\n</html>'
     _status = '303 See Other'
 
     def __call__(self, environ, start_response, **kwargs):
@@ -142,7 +154,7 @@ class BadGateway(Response):
     _status = "502 Bad Gateway"
 
 
-class HttpParameters():
+class HttpParameters(object):
     """GET or POST signature parameters for Redirect or POST-SimpleSign bindings
     because they are not contained in XML unlike the POST binding
     """
@@ -156,6 +168,7 @@ class HttpParameters():
         except KeyError:
             pass
 
+
 def extract(environ, empty=False, err=False):
     """Extracts strings in form data and returns a dict.
 
@@ -240,10 +253,7 @@ def unpack_redirect(environ):
 
 
 def unpack_post(environ):
-    try:
-        return dict([(k, v[0]) for k, v in parse_qs(get_post(environ))])
-    except Exception:
-        return None
+    return dict([(k, v[0]) for k, v in parse_qs(get_post(environ))])
 
 
 def unpack_soap(environ):
@@ -266,7 +276,7 @@ def unpack_artifact(environ):
 
 def unpack_any(environ):
     if environ['REQUEST_METHOD'].upper() == 'GET':
-    # Could be either redirect or artifact
+        # Could be either redirect or artifact
         _dict = unpack_redirect(environ)
         if "ID" in _dict:
             binding = BINDING_URI
@@ -307,7 +317,7 @@ def cookie_signature(seed, *parts):
     return sha1.hexdigest()
 
 
-def make_cookie(name, load, seed, expire=0, domain="",  path="",
+def make_cookie(name, load, seed, expire=0, domain="", path="",
                 timestamp=""):
     """
     Create and return a cookie
