@@ -43,7 +43,7 @@ class Request(object):
 
         # own copy
         self.xmlstr = xmldata[:]
-        logger.debug("xmlstr: %s" % (self.xmlstr,))
+        logger.debug("xmlstr: %s", self.xmlstr)
         try:
             self.message = self.signature_check(xmldata, origdoc=origdoc,
                                                 must=must,
@@ -55,15 +55,15 @@ class Request(object):
 
         if not self.message:
             logger.error("Response was not correctly signed")
-            logger.info(xmldata)
+            logger.info("Response: %s", xmldata)
             raise IncorrectlySigned()
 
-        logger.info("request: %s" % (self.message,))
+        logger.info("request: %s", self.message)
 
         try:
             valid_instance(self.message)
         except NotValid as exc:
-            logger.error("Not valid request: %s" % exc.args[0])
+            logger.error("Not valid request: %s", exc.args[0])
             raise
 
         return self
@@ -80,15 +80,21 @@ class Request(object):
         return issued_at > lower and issued_at < upper
 
     def _verify(self):
-        assert self.message.version == "2.0"
+        valid_version = "2.0"
+        if self.message.version != valid_version:
+            raise VersionMismatch(
+                "Invalid version {invalid} should be {valid}".format(
+                    invalid=self.message.version, valid=valid_version
+                )
+            )
+
         if self.message.destination and self.receiver_addrs and \
                 self.message.destination not in self.receiver_addrs:
-            logger.error("%s not in %s" % (self.message.destination,
-                                                self.receiver_addrs))
+            logger.error("%s not in %s", self.message.destination, self.receiver_addrs)
             raise OtherError("Not destined for me!")
 
-        assert self.issue_instant_ok()
-        return self
+        valid = self.issue_instant_ok()
+        return valid
 
     def loads(self, xmldata, binding, origdoc=None, must=None,
               only_valid_cert=False):
